
% WHAT IF? INPUT LAYER

% FIGURE 2

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% A row of exclamation marks indicates where values may need to be set, for
% example, the shape of the sampled distribution, sample size (1 field, 3 
% fields), or the terminal to gr cell ratio
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

% GENERATE MOSSY FIBRE RATES RECEIVED AS INPUT TO A RANK (A SAGITTAL ROW OF 
% 100 FIELDS)
% Randomly generates values that represent mossy fibre rates received as 
% input to a rank. The number of rates is equal to the number of mossy
% fibres which innervate a rank. The estimate of that number depends on the
% mossy fibre terminal to granule cell divergence ratio. The estimates are
% derived outside the program, in the mf code paper

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
M = 3645; % assumes mf terminal to gr cell divergence of 1:50
% M = 9112; % assumes 1:20
% M = 4554; % assumes 1:40
% M = 3036; % assumes 1:60

% The physiological range of mf firing rates in the behaving animal is 
% 50-300 Hz. A mean of 175 Hz is therefore mid-range.

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% SELECT A DISTIBUTION

% UNIFORM DISTRIBUTION OF RANDOM NUMBERS
% Randomly generates an array (row vector) of M integers in the range 
% 50-300, with a 'flat' distribution (i.e. all values are equally likely)
DIST = randi([50 300],1,M);
muDIST = mean(DIST);
sdDIST = std(DIST);

% % RANDOMLY SAMPLED FROM A NORMAL DISTRIBUTION WITH SPECIFIED MEAN AND SD
% % ONE PEAK
% m = 175;
% % sd = 12.5;
% sd = 25;
% % sd = 50;
% DIST = sd.*randn(1,M) + m; % array of M integers randomly drawn from a 
% % normal distribution with mean m and standard deviation sd
% muDIST = mean(DIST);
% sdDIST = std(DIST);
% % THREE PEAKS
% sd = 12.5;
% x = (M/10).*randi(4); xx = (M/10).*randi(4); xxx = M - (x + xx);
% a = randi(100)+50;b = randi(100)+125;c = randi(100)+200;
% aa = sd.*randn(1,round(x))+a;
% bb = sd.*randn(1,round(xx))+b;
% cc = sd.*randn(1,round(xxx))+c;
% DIST = [aa bb cc];
% muDIST = mean(DIST);
% sdDIST = std(DIST);
% % FIVE PEAKS
% sd = 12.5;
% a = randi(50)+50;b = randi(50)+100;c = randi(50)+150;d = randi(50)+200;e = randi(50)+250;
% aa = sd.*randn(1,M/5)+a;
% bb = sd.*randn(1,M/5)+b;
% cc = sd.*randn(1,M/5)+c;
% dd = sd.*randn(1,M/5)+d;
% ee = sd.*randn(1,M/5)+e;
% DIST = [aa bb cc dd ee];
% muDIST = mean(DIST);
% sdDIST = std(DIST);

% % DISCONTINUOUS DISTRIBUTION
% Hz = 50;
% DIST = [];
% for jj = 1:20
%     int = randi(20); DIST = [DIST, ones(1,round((int/20)*(M/10))) + Hz];
%     Hz = Hz + 12.5;
% end
% muDIST = mean(DIST);
% sdDIST = std(DIST);

% ** LOOP 1 **

% PREALLOCATE STORAGE SPACE
j = 100;
mfs_allflds = zeros(1,j); % Preallocates storage space for the array mfs_allflds
% % % rates_means = zeros(1,j); % not used
% % % rates_SDs = zeros(1,j); % not used
% % % copies_means = zeros(1,j); % not used 
% % % copies_SDs = zeros(1,j); % not used
rates_allflds = cell(1,j); % Preallocates storage space for a 1 x j cell array
copies_allflds = cell(1,j);

copiesmat_allflds = [];

% THE PROGRAM MUST REFLECT ANATOMICAL PARAMETERS THAT COULD ADD TO THE
% EFFECT OF CHANCE ON THE OUTCOME OF SAMPLING (I.E. LOCAL BIOLOGICAL
% MESSINESS)

% IN THIS SECTION/LOOP:
% (1) DERIVE THE (VARIABLE) NUMBER OF MFS THAT INNERVATE A FIELD. 
% (2) REPEAT FOR 100 FIELDS, STORED AS A 1 X 100 ARRAY. 
% (3) DERIVE THE NUMBER OF COPIES OF EACH SIGNAL RECEIVED BY A FIELD, 
% STORED AS A 1 X N ARRAY, WHERE N = THE NO. OF MFS WHICH INNERVATE THAT
% FIELD
% (4) DERIVE MF RATES RECEIVED BY EACH FIELD, RANDOMLY SAMPLED FROM DIST, 
% STORED AS A 1 x 100 CELL ARRAY. EACH CELL CONTAINS A COLUMN VECTOR OF 
% LENGTH N
% (5) THE SAME AS (4) BUT WITH COPIES ADDED, STORED AS A CELL ARRAY

i = 1;

while i <= j

    % STEP 1 - generates the ratio of cluster sizes in a simulated field

    % Cluster size ratio means: the ratio of mossy fibres (that contribute
    % at least 1 terminal to a field) that end in a cluster of 4 
    % terminals, 5, 6 and so on up to 12. the ratio is randomly generated. 
    % It is assumed each cluster size occurs with equal probability in the 
    % wider population.

    clust4 = []; clust5 = []; clust6 = []; clust7 = []; clust8 = []; 
    clust9 = []; clust10 = []; clust11 = []; clust12 = [];

    % !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    % The expected number of mossy fibres that innervate a field, derived
    % in the paper 
    expmf = 182.25; % assumes a mossy fibre terminal to granule cell divergence ratio of 1:50
%     expmf = 546.75; % ditto, but three fields
%     expmf = 455.6; % assumes a mossy fibre terminal to granule cell divergence ratio of 1:20
%     expmf = 227.7; % assumes a mossy fibre terminal to granule cell divergence ratio of 1:40
%     expmf = 151.8; % assumes a mossy fibre terminal to granule cell divergence ratio of 1:60

    count = 1;

    while count <= expmf % expmf is the expected number of mfs afferent to 
        % a field, an estimate derived in the mf code paper. Provides a  
        % starting point to generate field data. 
        num = randi([4 12],1,1);
        if num == 4
            clust4 = [clust4 1];
            tot4 = sum(clust4); % the number of 4-terminal clusters
        elseif num == 5
            clust5 = [clust5 1];
            tot5 = sum(clust5); % the number of 5-terminal clusters
        elseif num == 6
            clust6 = [clust6 1];
            tot6 = sum(clust6); % etc
        elseif num == 7
            clust7 = [clust7 1];
            tot7 = sum(clust7); 
        elseif num == 8
            clust8 = [clust8 1];
            tot8 = sum(clust8); 
        elseif num == 9
            clust9 = [clust9 1];
            tot9 = sum(clust9);
        elseif num == 10
            clust10 = [clust10 1];
            tot10 = sum(clust10); 
        elseif num == 11
            clust11 = [clust11 1];
            tot11 = sum(clust11); 
        else
            clust12 = [clust12 1];
            tot12 = sum(clust12);
        end

        count = count + 1;
    end

    % tot4:tot5:tot6...tot12 is the randomly generated ratio of cluster 
    % sizes
    ratios_sizes = [tot4 tot5 tot6 tot7 tot8 tot9 tot10 tot11 tot12];
    
    % STEP 2 - derives the ratio of terminals in the simulated field 
    % contributed by each cluster size

    % The ratio of cluster sizes is weighted to reflect the greater chance
    % that large clusters ontribute more terminals to a field. The expected
    % ratio of the number of terminals contributed by each cluster size is
    % derived in the main text.  

    exp_ratios_terminals = [22 25 27 28 28 28 28 28 28]; % the expected 
    % ratio of terminals contributed to a field by each cluster size
    % (derived in the mf code paper)
    ratios_terminals = ratios_sizes.*exp_ratios_terminals; % the randomly 
    % generated ratio of terminals per cluster size in the simulated field

    % STEP 3 - derives the number of terminals contributed to the simulated 
    % field by each cluster size 

    % Ratios are converted to fractions. The number of terminals 
    % contributed by each cluster size is the fraction contributed by that
    % size multiplied by the total number of terminals per field (derived
    % in the main text - depends on the terminal to granule cell divergence
    % ratio)

    fractions = ratios_terminals./sum(ratios_terminals); % the fraction of 
    % terminals contributed by each cluster size - what fraction is
    % contributed by mossy fibres ending in 4 terminals, 5 terminals, 6 and 
    % so on
    % !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    terminals = 700; % the total number of terminals per field 
%     terminals = 2100; % three fields
%     terminals = 1750; % one field, terminal to gr cell divergence of 1:20
    sum_each_size = round(fractions.*terminals); % the total number of 
    % terminals contributed by each cluster size

    % STEP 4 - derives the number of mossy fibres that terminate as each 
    % cluster size, which contribute at least one terminal to the simulated
    % field, and from that the number that innervate the field

    % For cluster size m, say, there are associated probabilities that all
    % terminals are inside field limits, m - 1 terminals are inside, m - 2
    % and so on down to 1. The function 'split' randomly generates the 
    % result of straddling field limits, reflecting that smaller clusters 
    % are more likely to have all terminals inside.
    
    % 'mfsn' is the number of mossy fibres which terminate in a cluster of 
    % size n (which have at least one terminal inside the simulated field). 
    % 'arrayn' is a row vector of values in the range 1 to n. Each value
    % corresponds to a mossy fibre, and represents the number of terminals
    % of that cell which are inside the field. Accordingly, mfsn is
    % equal to numel(arrayn).
    
    [mfs4,array4] = split(sum_each_size(1),4);
    [mfs5,array5] = split(sum_each_size(2),5);
    [mfs6,array6] = split(sum_each_size(3),6);
    [mfs7,array7] = split(sum_each_size(4),7);
    [mfs8,array8] = split(sum_each_size(5),8);
    [mfs9,array9] = split(sum_each_size(6),9);
    [mfs10,array10] = split(sum_each_size(7),10);
    [mfs11,array11] = split(sum_each_size(8),11);
    [mfs12,array12] = split(sum_each_size(9),12);

    % STEP 5 - derives the no. of mossy fibres with one terminal inside the
    % simulated field, two, three and so on.

    % Generates the outputs of section 1. these are:
    % (i) sample size i.e. the number of mfs that innervate a simulated field
    % (ii) the number of them that contribute 1 terminal to the field, 2
    % terminals, 3 and so on up to 12

    allmfs = [mfs4 mfs5 mfs6 mfs7 mfs8 mfs9 mfs10 mfs11 mfs12];
    % *********************************************************************
    mfs_1fld = sum(allmfs); % TOTAL MFS AFFERENT TO THE SIMULATED FIELD - 
    % THIS PROVIDES SAMPLE SIZE WHEN MF RATES ARE SAMPLED
    % *********************************************************************
    
    allarrays = [array4 array5 array6 array7 array8 array9 array10 array11 array12];
    terms_1fld = sum(allarrays); % A check - total number of terminals in  
    % the simulated field. Should equal 'terminals'
    
    % *********************************************************************
    mfs_allflds(i) = mfs_1fld; % BUILDS INTO AN ARRAY. VALUES REPRESENT THE 
    % NUMBER OF MFS AFFERENT TO EACH FIELD (IN A ROW OF j FIELDS)
    % *********************************************************************
    
    av_mfs_1fld = mean(mfs_allflds); % A check - av_mfs_1fld is mean sample
    % size - this should be somewhere around expmf, the expected no. of mfs 
    % afferent to a field
    
    % A field receives multiple copies of most signals because most mossy
    % fibres contribute more than one terminal. Therefore, copies are added to
    % a sample in the simulation. The number of copies of each signal depends  
    % on relevant probabilities (of each cluster size, of straddling, and of 
    % the result of straddling), and is randomly generated. This has the form 
    % of an array of factors with a number of elements equal to each sample 
    % size. 

    t1 = sum(allarrays == 1); unos = ones(1,t1); % the number of mossy 
    % that contribute one terminal to the simulated field and an array of 
    % ones of that length
    t2 = sum(allarrays == 2); twos = 2.*ones(1,t2); % the number of mossy 
    % fibres that contribute two terminals to the simulated field and an
    % array of twos of that length
    t3 = sum(allarrays == 3); threes = 3.*ones(1,t3); % and so on
    t4 = sum(allarrays == 4); fours = 4.*ones(1,t4);
    t5 = sum(allarrays == 5); fives = 5.*ones(1,t5);
    t6 = sum(allarrays == 6); sixes = 6.*ones(1,t6);
    t7 = sum(allarrays == 7); sevens = 7.*ones(1,t7);
    t8 = sum(allarrays == 8); eights = 8.*ones(1,t8);
    t9 = sum(allarrays == 9); nines = 9.*ones(1,t9);
    t10 = sum(allarrays == 10); tens = 10.*ones(1,t10);
    t11 = sum(allarrays == 11); elevens = 11.*ones(1,t11);
    t12 = sum(allarrays == 12); twelves = 12.*ones(1,t12);

    % *********************************************************************
    copies = [unos twos threes fours fives sixes sevens eights nines tens...
        elevens twelves]; % AN ARRAY OF ONES, TWOS, THREES AND SO ON WHICH
    % REPRESENTS THE NUMBER OF COPIES OF EACH MF SIGNAL RECEIVED BY A
    % FIELD. NOTE: the maximum number of copies of a signal is 7 (because
    % that's the maximum number that fit is a field). t8-t12 have the
    % function of mopping up leftovers returned by 'split', if any.
    % *********************************************************************
    check = numel(copies); % should equal mfs_1fld
    
    % SAMPLE DIST AND ADD COPIES
    rates_1fld = datasample(DIST,mfs_1fld,'Replace',false)'; % A column of 
    % values reoresenting mf rates received by a field
    copies_1fld = repelem(rates_1fld,copies); % A longer column of values 
    % because it contains copies
    
    % *********************************************************************
    rates_allflds{i} = rates_1fld; % BUILDS INTO A 1 X j CELL ARRAY. EACH   
    % CELL CONTAINS A COLUMN OF VALUES, A RANDOM SAMPLE OF DIST, EQUAL TO THE 
    % NUMBER OF MFS AFFERENT TO THAT FIELD, REPRESENTING MF RATES RECEIVED 
    % AS INPUT
    % *********************************************************************
    
    % The next section of code adds or removes values so that copies_1fld
    % has exactly 700 elements (it's not more than one or two out either
    % way anyway). This is not unphysiological - there are supposed to be
    % 700 as that is the number of terminals in a field. It's redundant
    % really - it was written to test an idea that didn't work - but it
    % allows the data for all fields to be stored in a matrix, which might
    % be useful in a later figure
    
    y = copies_1fld;
    size = numel(y);
    if size > 700
        diff = size - 700;
        y = y(1:end - diff); % truncate by diff
    elseif size < 700
        diff = 700 - size;
        for k = 1:diff
            y = y';
            y = [y, y(1)]; % put a copy of 
            % the first element at the end; repeat diff times
            y = y';
        end
    end
    copies_1fld = y;
    
    % *********************************************************************
    copies_allflds{i} = copies_1fld; % DITTO (SEE ABOVE) BUT WITH COPIES 
    % ADDED. NOTE: VALUES STILL REPRESENT MF RATES, BUT THESE INCLUDE 
    % COPIES TO REFLECT THE FACT THAT A FIELD RECEIVES COPIES OF MOST 
    % SIGNALS
    % *********************************************************************
    
    copiesmat_allflds = [copiesmat_allflds copies_1fld]; % A 700 x j matrix.
    % Contains the same data as copies_allflds but in a matrix instead of
    % a cell array. Not used but may be useful in a later figure.
    
    i = i + 1;

end

WTF = rates_1fld
NUM = numel(WTF)

c = copies_allflds;
threeflds = [c{1}',c{2}',c{3}']; % Data in the first three cells of 
% copies_allflds are converted to an array so they will be accepted as 
% input to the histogram function

% The block of commented code was written to test an idea that didn't work
% but retained in case there is a later use for it

% randhalf_allflds = zeros(350,j); % A 350 x j matrix
% for i = 1:j
%     randhalf_allflds(:,i) = datasample(copiesmat_allflds(:,i),350);
% end
% extend_allflds = zeros(1400,j); % A 1400 x j matrix
% for i = 1:j
%     if i == 1
%         extend_allflds(:,i) = [randhalf_allflds(:,end)', copiesmat_allflds(:,i)', randhalf_allflds(:,i + 1)']';
%     elseif i == j
%         extend_allflds(:,i) = [randhalf_allflds(:,i - 1)', copiesmat_allflds(:,i)', randhalf_allflds(:,1)']';
%     else
%         extend_allflds(:,i) = [randhalf_allflds(:,i - 1)', copiesmat_allflds(:,i)', randhalf_allflds(:,i + 1)']';
%     end
% end

for i = 1:3
    Gdend_samples = zeros(4,50);
    for k = 1:50
        Gdend_samples(:,k) = datasample(copies_allflds{i},4)'; % NOTE: granule 
        % cells are sampling mossy fibre terminals, not mossy fibre rates
    end
    if i == 1
        Gdend_fld1 = Gdend_samples;
    elseif i == 2
        Gdend_fld2 = Gdend_samples;
    elseif i == 3
        Gdend_fld3 = Gdend_samples;
        Gdend_3flds = [Gdend_fld1 Gdend_fld2 Gdend_fld3];
        Gdendmeans = mean(Gdend_3flds);
%         for j = 1:30
%             Gcells(:,j) = datasample(Gdendmeans,5,'Replace',false)';
%         end
        Gcells = reshape(Gdendmeans,5,30); % The same result as randomly  
        % sampling Gdendmeans 30 times without replacement, sample size 5, 
        % storing each sample as a column of a new matrix. It achieves an
        % equivalent of removing sampled values from the initial
        % population, so they can't be picked twice
        Gcellmeans = mean(Gcells);
        GLOMmeans = zeros(1,700);
        k = 1;
        while k <= 700
            glom = datasample(Gcellmeans,randi([8 12],1));
            glommean = mean(glom);
            GLOMmeans(k) = glommean;
            k = k + 1;
        end
    end
end
            
% % %     rates_means(i) = mean(sample); % not used
% % %     copies_means(i) = mean(copies_added); % not used

% FIGURE PANELS

opengl software
figure;

subplot(1,5,1)
histogram(DIST,'BinWidth',5,'facecolor',[.82 .7 .55]);%,'facealpha',.4)
xlabel('Hz');ylabel('');title('mf rates (5Hz bins)');
xlim([50 300]);xticks([100 200 300]);%ylim([0 300]);
set(gca,'FontSize',11)

hold on 

histogram(threeflds,'BinWidth',5,'facecolor',[.635 .078 .184])%[.4 0 0])
% histogram(samplesarray,'BinWidth',5,'facecolor',[.4 0 0])

subplot(1,5,2)
x = 1:150;
plot(x,Gdendmeans,'LineWidth',.9)
xlabel('dendrites');ylabel('sample means (Hz)');title('G-cells');
xlim([1 150]);ylim([0 300]);xticks([1 50 100 150])
set(gca,'FontSize',11)

subplot(1,5,3)
x = 1:30;
plot(x,Gcellmeans,'LineWidth',1.2)
xlabel('somata');ylabel('averaged sample means (Hz)');title('G-cells');
xlim([1 30]);ylim([0 300]);xticks([1 10 20 30])
set(gca,'FontSize',11)

subplot(1,5,4)
x = 1:700;
plot(x,GLOMmeans,'LineWidth',1)
xlabel('glomeruli');ylabel('GABA calibrated in Hz');title('output');
xlim([1 700]);xticks([1 350 700]);ylim([0 300])
set(gca,'FontSize',11)

meanarray = repelem(muDIST,700); A = meanarray;
SDarray = repelem(sdDIST,700); S = SDarray;
hold on
s = patch([x fliplr(x)], [A-S  fliplr(A+S)], [.82 .7 .55],'EdgeColor',[.57 .5 .32]);
alpha(s,.3)

subplot(1,5,5)
histogram(GLOMmeans,'BinWidth',5,'facecolor',[0 .447 .741]);%[.29 .66 .76])%,'facealpha',.4)
xlabel('Hz');ylabel('');title('output (5Hz bins)');xlim([50 300]);xticks([100 200 300]);%ylim([0 300]);
set(gca,'FontSize',11)
